@@ -1,5 +1,5 @@
 from __future__ import annotations
-
+import inspect
 import copy
 from typing import Iterator
 from uuid import uuid4
@@ -30,6 +30,7 @@ class Project(object):
         self.seperator_status = True
         self.seperator = ","
 
+
     @property
     def changed(self) -> bool:
         def check_data():
@@ -212,6 +213,12 @@ class PropertySet(Hirarchy):
             self.identifier = str(uuid4())
         self.changed = True
 
+    def __lt__(self,other):
+        if isinstance(other,PropertySet):
+            return self.name< other.name
+        else:
+            return self.name < other
+
     @property
     def is_predefined(self) -> bool:
         if self.object is None:
@@ -352,6 +359,12 @@ class Attribute(Hirarchy):
         text = f"{self.property_set.name} : {self.name} = {self.value}"
         return text
 
+    def __lt__(self, other):
+        if isinstance(other,Attribute):
+            return self.name < other.name
+        else:
+            return self.name < other
+
     @property
     def revit_name(self) -> str:
         return self._revit_name
@@ -495,6 +508,7 @@ class Object(Hirarchy):
         self._property_sets: list[PropertySet] = list()
         self._ident_attrib = ident_attrib
         self._aggregations: set[Aggregation] = set()
+        self.custom_attribues = {}
 
         if ifc_mapping is None:
             self._ifc_mapping = {"IfcBuildingElementProxy"}
@@ -511,6 +525,10 @@ class Object(Hirarchy):
     def __str__(self):
         return f"Object {self.name}"
 
+    def __lt__(self, other:Object):
+        return self.ident_value < other.ident_value
+
+
     @property
     def ifc_mapping(self) -> set[str]:
         return self._ifc_mapping
